/*
 * Copyright (C) 2003-2004 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
// @version $Id: SyncMLDMAcc.cpp,v 1.0 2004/04/27 11:21:00 magi Exp $
//
#include "spdm/win32/SyncMLDMConfig.h"
#include "spdm/win32/SyncMLDMAcc.h"


SyncMLDMAcc::SyncMLDMAcc(ManagementNode* node) {
    LOG.debug(TEXT("in SyncMLDMAcc"));
    readFromDM(node);    
}



BOOL SyncMLDMAcc::readFromDM(ManagementNode* node) {
    
    BOOL ret = FALSE;
    int  err = 0;
    if (node == NULL) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("SyncMLDMAcc configuration NULL"));
        LOG.error(lastErrorMsg);
        goto finally;
    }
    wchar_t buf[INT_VALUESIZE]; // big enough for everything

    err = node->getPropertyValue(PROPERTY_USER, buf, INT_VALUESIZE, FALSE );
    setUsername(buf);
    if (err != 0) goto finally;


    node->getPropertyValue(PROPERTY_CLIENTPW,   buf, INT_VALUESIZE, FALSE );
    setClientPW(buf);
    if (err != 0) goto finally;
    
    node->getPropertyValue(PROPERTY_ADDR,       buf, INT_VALUESIZE, FALSE );
    setAddr(buf);
    if (err != 0) goto finally;

    node->getPropertyValue(PROPERTY_ADDRTYPE,   buf, INT_VALUESIZE, FALSE );
    setAddrType(buf);
    if (err != 0) goto finally;

    node->getPropertyValue(PROPERTY_PORTNUMBER, buf, INT_VALUESIZE, FALSE );
    setPortNumber(buf);
    if (err != 0) goto finally;

    node->getPropertyValue(PROPERTY_SERVERID,   buf, INT_VALUESIZE, FALSE );
    setServerId(buf);
    if (err != 0) goto finally;

    node->getPropertyValue(PROPERTY_SERVERPW,   buf, INT_VALUESIZE, FALSE );
    setServerPW(buf);
    if (err != 0) goto finally;

    node->getPropertyValue(PROPERTY_CLIENTNONCE,buf, INT_VALUESIZE, FALSE );
    setClientNonce(buf);
    if (err != 0) goto finally;

    if (lastErrorCode != 0) { // node not found
        wsprintf(lastErrorMsg, TEXT("SyncMLDMAcc configuration not found"));
        LOG.error(lastErrorMsg);
        goto finally;
    }

    ret = TRUE;
    
finally:
    if (err != 0) {
        wsprintf(lastErrorMsg, TEXT("SyncMLDMAcc configuration properties not found"));
        LOG.error(lastErrorMsg);
        lastErrorCode = -1;
    }

    return ret;

}


BOOL SyncMLDMAcc::saveToDM(ManagementTree *mt, wchar_t* nameDMAcc) { 
    
    wchar_t context [DIM_CONTEXT  ]; 
    wchar_t buf     [INT_VALUESIZE];
    wsprintf(context, TEXT("%s/%s/%s"), DM_ROOT_CONTEXT, DM_ACC_CONTEXT, nameDMAcc);
    

    ManagementNode* node = mt->getManagementNode(context);
    
    wsprintf(buf, TEXT("%s"), getClientNonce());
    node->setPropertyValue(PROPERTY_CLIENTNONCE, buf);
    
    delete node;

    return TRUE;

}



wchar_t* SyncMLDMAcc::getUsername() {
    return username;
}

void SyncMLDMAcc::setUsername(wchar_t* p) {
    wcsncpy(username, p, DIM_USER);
    username[DIM_USER -1] = 0;
    
}

wchar_t* SyncMLDMAcc::getClientPW() {
    return clientPW;
}

void SyncMLDMAcc::setClientPW(wchar_t* p) {
    wcsncpy(clientPW, p, DIM_CLIENTPW);
    clientPW[DIM_CLIENTPW -1] = 0;
    
}

wchar_t* SyncMLDMAcc::getAddr() {
    return addr;
}
void SyncMLDMAcc::setAddr(wchar_t* p) {
    wcsncpy(addr, p, DIM_ADDR);
    addr[DIM_ADDR -1] = 0;
    
}

wchar_t* SyncMLDMAcc::getAddrType() {
    return addrType;
}
void SyncMLDMAcc::setAddrType(wchar_t* p) {
    wcsncpy(addrType, p, DIM_ADDRTYPE);
    addrType[DIM_ADDRTYPE -1] = 0;
    
}

wchar_t* SyncMLDMAcc::getPortNumber() {
    return portNumber;
}
void SyncMLDMAcc::setPortNumber(wchar_t* p) {
    wcsncpy(portNumber, p, DIM_PORTNUMBER);
    portNumber[DIM_PORTNUMBER -1] = 0;
    
}

wchar_t* SyncMLDMAcc::getServerId() {
    return serverId;
}
void SyncMLDMAcc::setServerId(wchar_t* p) {
    wcsncpy(serverId, p, DIM_SERVERID);
    serverId[DIM_SERVERID -1] = 0;
    
}

wchar_t* SyncMLDMAcc::getServerPW() {
    return serverPW;
}
void SyncMLDMAcc::setServerPW(wchar_t* p) {
    wcsncpy(serverPW, p, DIM_SERVERPW);
    serverPW[DIM_SERVERPW -1] = 0;
    
}

wchar_t* SyncMLDMAcc::getClientNonce() {
    return clientNonce;
}
void SyncMLDMAcc::setClientNonce(wchar_t* p) {
    wcsncpy(clientNonce, p, DIM_CLIENTNONCE);
    clientNonce[DIM_CLIENTNONCE -1] = 0;
    
}

