/*
 * Copyright (C) 2003-2004 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Stefano Fornari
// @version $Id: Win32ManagementNode.cpp,v 1.7 2004/04/15 16:44:16 magi Exp $
//

#include <stdio.h>  // to be removed
#include <stdlib.h>

/*
 * Win32 implementation of ManagementNode
 */

#include "common/fscapi.h"
#include "spdm/common/Utils.h"
#include "spdm/common/Constants.h"
#include "spdm/common/ManagementNode.h"
#include "spdm/win32/Win32ManagementNode.h"
#include "common/Log.h"

Win32ManagementNode::Win32ManagementNode(wchar_t* context, wchar_t* name) : ManagementNode(context, name) {
    wsprintf(fullContext, TEXT("Software/%s/%s"), context, name);
    convertSlashes(fullContext);
 }

void Win32ManagementNode::getPropertyValue(const wchar_t* property, wchar_t* buf, int size) {
    
    getPropertyValue(property, buf, size, TRUE);
}

/*
 * Returns the value of the given property
 *
 * @param property - the property name
 * @param buf - the buffer for the value
 * @param len - the buffer size
 */
int Win32ManagementNode::getPropertyValue(const wchar_t* property, wchar_t* buf, int size, BOOL isToCreate) {
    HKEY key = NULL;
    ULONG dim = size;
    DWORD res;	

    long err = 0; // 0 success, != 0 error in retrieve value


    if (isToCreate == FALSE) 
        RegOpenKeyEx(
             HKEY_LOCAL_MACHINE,
             fullContext,
             0,
             KEY_READ,
             &key
        );
    else
        RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            fullContext,
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
        );

    if (key == NULL || key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;
    }


    err = RegQueryValueEx(
        key,
        property,
        NULL,
        NULL,  // we currently support only strings
        (UCHAR*)buf,
        &dim
    );
/*    
    LPVOID lpMsgBuf;
    if (!FormatMessage( 
        FORMAT_MESSAGE_ALLOCATE_BUFFER | 
        FORMAT_MESSAGE_FROM_SYSTEM |
        FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        err,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
        (LPTSTR) &lpMsgBuf,
        0,
        NULL ))
    {
       // Handle the error.
       return err;
    }
    // Process any inserts in lpMsgBuf.
    // ...
    // Display the string.
    MessageBox( NULL, (LPCTSTR)lpMsgBuf, TEXT("Error"), MB_OK | MB_ICONINFORMATION );
    // Free the buffer.
    LocalFree( lpMsgBuf );
*/
    if (err == ERROR_SUCCESS) {
        ; //MessageBox(NULL,  TEXT("Correct"), TEXT("getConfigParameter"), MB_OK);
    } else {		
        ; //MessageBox(NULL,  TEXT("Error"), TEXT("getConfigParameter"), MB_OK);
        
        wsprintf(buf, TEXT(""));
    }

    finally:

    if (key != 0) {
       RegCloseKey(key);
    }
    return err;
}


/*
 * Returns the value of the given property
 *
 * @param name - the key name
 * @param buf - the buffer for the value
 * @param len - the buffer size
 */
void Win32ManagementNode::getChildrenName(const wchar_t* name, wchar_t* buf, int size, BOOL isToCreate) {
    HKEY key = NULL;
    ULONG dim = size;
    DWORD res;	
    DWORD numSubKeys;
    DWORD numValues;
    wchar_t child[DIM_MANAGEMENT_PATH];
    ULONG dimension = DIM_MANAGEMENT_PATH;

    long err = 0;


    if (isToCreate == FALSE) 
        RegOpenKeyEx(
             HKEY_LOCAL_MACHINE,
             fullContext,
             0,
             KEY_READ,
             &key
        );
    else
        RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            fullContext,
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
        );

    if (key == NULL || key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;
    }

    
    err = RegQueryInfoKey( key,
        NULL,
        NULL,
        NULL,
        &numSubKeys,
        NULL,
        NULL,
        &numValues,
        NULL,
        NULL,
        NULL,
        NULL);
        

   
    if (numSubKeys != 0) {
        for (unsigned int i = 0; i < numSubKeys ; i++) {
            wmemset(child, 0, DIM_MANAGEMENT_PATH);
            dimension = DIM_MANAGEMENT_PATH;
            if (RegEnumKeyEx(key, i, child, &dimension, NULL, NULL, NULL, NULL) == ERROR_NO_MORE_ITEMS) {
                          
                break;
            };               
            wcscat(buf, child);   
            wcscat(buf, TEXT("/")); 
        }
        buf[wcslen(buf) - 1] = 0;
    
    }
        


    finally:

    if (key != 0) {
       RegCloseKey(key);
    }
}

/*
* If key and property don't exist, it creates it
*/
void Win32ManagementNode::setPropertyValue(const wchar_t* property, wchar_t* value) {
    setPropertyValue(property,value, TRUE);
}

/*
* If BOOL is True, it creates key, if False it doesn't. If key arleady exists, method sets value to the property.
* If property doesn't exist, method creates it.
*/

void Win32ManagementNode::setPropertyValue(const wchar_t* property, wchar_t* value, BOOL isKeyToCreate) {
    setPropertyValue(property,value, isKeyToCreate, TRUE);
}
/*
 * Sets a property value.
 * 
 * If BOOL isKeyToCreate is TRUE, it creates key, if False it doesn't. If key arleady exists, method sets value to the property.
 * If isPropertyToCreate is TRUE and property doesn't exist, method creates it. Otherwise it doesn't
 */
void Win32ManagementNode::setPropertyValue(const wchar_t* property, wchar_t* value, BOOL isKeyToCreate, BOOL isPropertyToCreate) {
    HKEY key;
    DWORD res;
    int resultAction = ERROR_SUCCESS;
    wchar_t previousValue [DIM_MANAGEMENT_PATH];
    wmemset(previousValue, 0, DIM_MANAGEMENT_PATH);

    if (isKeyToCreate == FALSE) 
        RegOpenKeyEx(
             HKEY_LOCAL_MACHINE,
             fullContext,
             0,
             KEY_READ | KEY_WRITE,
             &key
        );
    else
        RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            fullContext,
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
        );


    if (key == NULL || key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;
    }
    
    // for DM implementation
    resultAction = getPropertyValue(property, previousValue, DIM_MANAGEMENT_PATH, FALSE);
    
     if (isPropertyToCreate == FALSE && resultAction != ERROR_SUCCESS)  { // no property is found and not create it
        
        lastErrorCode = resultAction;
        goto finally;
    }

    if (previousValue == NULL || wcscmp(previousValue, value) == 0) {
        lastErrorCode = ERR_EQUALS_VALUES; //418
        wsprintf(lastErrorMsg, TEXT("Value arleady exists:"), previousValue);
        goto finally;
    }
    
   

    RegSetValueEx(
        key,
        property,
        NULL,
        REG_SZ,  // we currently support only strings
        (UCHAR*)value,
        (wcslen(value)+1)*sizeof(wchar_t*)
    );


    finally:

    if (key != 0) {
       RegCloseKey(key);
    }
}

/*
 * Returns this node's children.
 *
 * The ManagementNode objects are created with the new operator and
 * must be discarded by the caller with the operator delete.
 *
 * @param children - the buffer where ManagementNode* must be stored
 * @param size - the size of the children buffer (number of ManagementNode*) in
 *               input; the number of children in output
 */
void Win32ManagementNode::getChildren(ManagementNode** children, int* size) {
    int i;
    ULONG dim = DIM_MANAGEMENT_PATH;
    HKEY key;
    wchar_t child[DIM_MANAGEMENT_PATH];
    wchar_t fullName[DIM_MANAGEMENT_PATH];
    DWORD res;

/*
    RegOpenKeyEx(
         HKEY_LOCAL_MACHINE,
         fullContext,
         0,
         KEY_READ,
         &key
    );
*/
    RegCreateKeyEx(
        HKEY_LOCAL_MACHINE,
        fullContext,
        0,
        NULL,
        REG_OPTION_NON_VOLATILE,
        KEY_ALL_ACCESS,
        NULL,
        &key,
        &res
    );

    //
    // First of all we need to know how many

    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;
    }

    getFullName(fullName, DIM_MANAGEMENT_PATH);
    for (i=0; i<*size; ++i) {
        if (RegEnumKeyEx(key, i, child, &dim, NULL, NULL, NULL, NULL) == ERROR_NO_MORE_ITEMS) {
            break;
        };

        children[i] = new Win32ManagementNode(fullName, child);
    }

    *size = i;


    finally:

    if (key != 0) {
       RegCloseKey(key);
    }
}

/*
 * Returns how many children belong to this node.
 * A negative number means an error
 *
 */
int Win32ManagementNode::getChildrenCount() {
    HKEY key;
    ULONG howMany = 0;

    RegOpenKeyEx(
         HKEY_LOCAL_MACHINE,
         fullContext,
         0,
         KEY_READ,
         &key
    );

    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);

        goto finally;
    }

    RegQueryInfoKey(
        key, NULL, NULL, NULL, &howMany, NULL, NULL, NULL, NULL, NULL, NULL, NULL
    );


    finally:

    if (key != 0) {
       RegCloseKey(key);
    }

    return howMany;
}

/*
 * Returns a child node of the ManagementNode.
 *
 * The child ManagementNode object is created with the new operator
 * must be discarded by the caller with the operator delete.
 *
 * @param node - subnode name
 */
ManagementNode* Win32ManagementNode::getChild(const wchar_t* node) {
    wchar_t nodeName   [DIM_MANAGEMENT_PATH];
    wchar_t nodeContext[DIM_MANAGEMENT_PATH];
    wchar_t fullName   [DIM_MANAGEMENT_PATH];


    getNodeName(node, nodeName, DIM_MANAGEMENT_PATH);

    getNodeContext(node, nodeContext, DIM_MANAGEMENT_PATH);


    wcscpy(fullName, context);
    wcscat(fullName, TEXT("/"));
    wcscat(fullName, nodeContext);

    return new Win32ManagementNode(fullName, nodeName);
}


/*
 * Device management contexts are expressed as slash '/' separated
 * context path. This function converts the '/' chars in '\\' as
 * accepted by the registry API.
 *
 * @param str - the string to converted
 *
 */
void Win32ManagementNode::convertSlashes(wchar_t* str) {
    int i, l;

    l = wcslen(str);

    for (i=0; i<l; ++i) {
        if (str[i] == L'/') {
            str[i] = L'\\';
        }
    }
}